package de.uniulm.semweb;

import java.util.HashSet;
import java.util.Set;

import org.semanticweb.owlapi.apibinding.OWLManager;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLClassExpression;
import org.semanticweb.owlapi.model.OWLClassExpressionVisitorEx;
import org.semanticweb.owlapi.model.OWLDataAllValuesFrom;
import org.semanticweb.owlapi.model.OWLDataExactCardinality;
import org.semanticweb.owlapi.model.OWLDataFactory;
import org.semanticweb.owlapi.model.OWLDataHasValue;
import org.semanticweb.owlapi.model.OWLDataMaxCardinality;
import org.semanticweb.owlapi.model.OWLDataMinCardinality;
import org.semanticweb.owlapi.model.OWLDataSomeValuesFrom;
import org.semanticweb.owlapi.model.OWLObjectAllValuesFrom;
import org.semanticweb.owlapi.model.OWLObjectComplementOf;
import org.semanticweb.owlapi.model.OWLObjectExactCardinality;
import org.semanticweb.owlapi.model.OWLObjectHasSelf;
import org.semanticweb.owlapi.model.OWLObjectHasValue;
import org.semanticweb.owlapi.model.OWLObjectIntersectionOf;
import org.semanticweb.owlapi.model.OWLObjectMaxCardinality;
import org.semanticweb.owlapi.model.OWLObjectMinCardinality;
import org.semanticweb.owlapi.model.OWLObjectOneOf;
import org.semanticweb.owlapi.model.OWLObjectSomeValuesFrom;
import org.semanticweb.owlapi.model.OWLObjectUnionOf;

public class NNFConverter {

    protected OWLDataFactory dataFactory=OWLManager.getOWLDataFactory(); 
    protected PositiveNNFVisitor positiveNNFVisitor=new PositiveNNFVisitor();
    protected NegativeNNFVisitor negativeNNFVisitor=new NegativeNNFVisitor();
    
    public OWLClassExpression convertToNNF(OWLClassExpression ce) {
        return ce.accept(positiveNNFVisitor);
    }
    
    protected class PositiveNNFVisitor implements OWLClassExpressionVisitorEx<OWLClassExpression> {
    
        @Override
        public OWLClassExpression visit(OWLClass ce) {
            return ce;
        }
            
        @Override
        public OWLClassExpression visit(OWLObjectIntersectionOf ce) {
            Set<OWLClassExpression> newConjuncts=new HashSet<OWLClassExpression>();
            for (OWLClassExpression conjunct : ce.getOperands()) {
                OWLClassExpression nnfConjunct=conjunct.accept(this);
                newConjuncts.add(nnfConjunct);
            }
            return dataFactory.getOWLObjectIntersectionOf(newConjuncts);
        }
    
        @Override
        public OWLClassExpression visit(OWLObjectUnionOf ce) {
            Set<OWLClassExpression> newDisjuncts=new HashSet<OWLClassExpression>();
            for (OWLClassExpression disjunct : ce.getOperands()) {
                OWLClassExpression nnfDisjunct=disjunct.accept(this);
                newDisjuncts.add(nnfDisjunct);
            }
            return dataFactory.getOWLObjectUnionOf(newDisjuncts);
        }
    
        @Override
        public OWLClassExpression visit(OWLObjectComplementOf ce) {
            OWLClassExpression negated=ce.getOperand();
            return negated.accept(negativeNNFVisitor);
        }
    
        @Override
        public OWLClassExpression visit(OWLObjectSomeValuesFrom ce) {
            OWLClassExpression filler=ce.getFiller().accept(this);
            return dataFactory.getOWLObjectSomeValuesFrom(ce.getProperty(),filler);
        }
    
        @Override
        public OWLClassExpression visit(OWLObjectAllValuesFrom ce) {
            OWLClassExpression filler=ce.getFiller().accept(this);
            return dataFactory.getOWLObjectAllValuesFrom(ce.getProperty(),filler);
        }
    
        @Override
        public OWLClassExpression visit(OWLObjectHasValue ce) {
            // optional, nicht erforderlich zum Bestehen der bung
            return ce;
        }
    
        @Override
        public OWLClassExpression visit(OWLObjectMinCardinality ce) {
         // optional, nicht erforderlich zum Bestehen der bung
            return ce;
        }
    
        @Override
        public OWLClassExpression visit(OWLObjectExactCardinality ce) {
         // optional, nicht erforderlich zum Bestehen der bung
            return ce;
        }
    
        @Override
        public OWLClassExpression visit(OWLObjectMaxCardinality ce) {
         // optional, nicht erforderlich zum Bestehen der bung
            return ce;
        }
    
        @Override
        public OWLClassExpression visit(OWLObjectHasSelf ce) {
         // optional, nicht erforderlich zum Bestehen der bung
            return ce;
        }
    
        @Override
        public OWLClassExpression visit(OWLObjectOneOf ce) {
         // optional, nicht erforderlich zum Bestehen der bung
            return ce;
        }
    
        @Override
        public OWLClassExpression visit(OWLDataSomeValuesFrom ce) {
         // optional, nicht erforderlich zum Bestehen der bung
            return ce;
        }
    
        @Override
        public OWLClassExpression visit(OWLDataAllValuesFrom ce) {
         // optional, nicht erforderlich zum Bestehen der bung
            return ce;
        }
    
        @Override
        public OWLClassExpression visit(OWLDataHasValue ce) {
         // optional, nicht erforderlich zum Bestehen der bung
            return ce;
        }
    
        @Override
        public OWLClassExpression visit(OWLDataMinCardinality ce) {
         // optional, nicht erforderlich zum Bestehen der bung
            return ce;
        }
    
        @Override
        public OWLClassExpression visit(OWLDataExactCardinality ce) {
         // optional, nicht erforderlich zum Bestehen der bung
            return ce;
        }
    
        @Override
        public OWLClassExpression visit(OWLDataMaxCardinality ce) {
         // optional, nicht erforderlich zum Bestehen der bung
            return ce;
        }
    }
    
    protected class NegativeNNFVisitor implements OWLClassExpressionVisitorEx<OWLClassExpression> {
        
        @Override
        public OWLClassExpression visit(OWLClass ce) {
            if (ce.isOWLThing())
                return dataFactory.getOWLNothing();
            if (ce.isOWLNothing())
                return dataFactory.getOWLThing();
            return dataFactory.getOWLObjectComplementOf(ce);
        }
            
        @Override
        public OWLClassExpression visit(OWLObjectIntersectionOf ce) {
            Set<OWLClassExpression> newDisjuncts=new HashSet<OWLClassExpression>();
            for (OWLClassExpression conjunct : ce.getOperands())
                newDisjuncts.add(conjunct.accept(this));
            return dataFactory.getOWLObjectUnionOf(newDisjuncts);
        }
    
        @Override
        public OWLClassExpression visit(OWLObjectUnionOf ce) {
            Set<OWLClassExpression> newConjuncts=new HashSet<OWLClassExpression>();
            for (OWLClassExpression disjunct : ce.getOperands())
                newConjuncts.add(disjunct.accept(this));
            return dataFactory.getOWLObjectIntersectionOf(newConjuncts);
        }
    
        @Override
        public OWLClassExpression visit(OWLObjectComplementOf ce) {
            OWLClassExpression negated=ce.getOperand();
            return negated.accept(positiveNNFVisitor);
        }
    
        @Override
        public OWLClassExpression visit(OWLObjectSomeValuesFrom ce) {
            OWLClassExpression nnfFiller=ce.getFiller().accept(this);
            return dataFactory.getOWLObjectAllValuesFrom(ce.getProperty(), nnfFiller);
        }
    
        @Override
        public OWLClassExpression visit(OWLObjectAllValuesFrom ce) {
            OWLClassExpression nnfFiller=ce.getFiller().accept(this);
            return dataFactory.getOWLObjectSomeValuesFrom(ce.getProperty(), nnfFiller);
        }
    
        @Override
        public OWLClassExpression visit(OWLObjectHasValue ce) {
         // optional, nicht erforderlich zum Bestehen der bung
            return ce;
        }
    
        @Override
        public OWLClassExpression visit(OWLObjectMinCardinality ce) {
         // optional, nicht erforderlich zum Bestehen der bung
            return ce;
        }
    
        @Override
        public OWLClassExpression visit(OWLObjectExactCardinality ce) {
         // optional, nicht erforderlich zum Bestehen der bung
            return ce;
        }
    
        @Override
        public OWLClassExpression visit(OWLObjectMaxCardinality ce) {
         // optional, nicht erforderlich zum Bestehen der bung
            return ce;
        }
    
        @Override
        public OWLClassExpression visit(OWLObjectHasSelf ce) {
         // optional, nicht erforderlich zum Bestehen der bung
            return ce;
        }
    
        @Override
        public OWLClassExpression visit(OWLObjectOneOf ce) {
         // optional, nicht erforderlich zum Bestehen der bung
            return ce;
        }
    
        @Override
        public OWLClassExpression visit(OWLDataSomeValuesFrom ce) {
         // optional, nicht erforderlich zum Bestehen der bung
            return ce;
        }
    
        @Override
        public OWLClassExpression visit(OWLDataAllValuesFrom ce) {
         // optional, nicht erforderlich zum Bestehen der bung
            return ce;
        }
    
        @Override
        public OWLClassExpression visit(OWLDataHasValue ce) {
         // optional, nicht erforderlich zum Bestehen der bung
            return ce;
        }
    
        @Override
        public OWLClassExpression visit(OWLDataMinCardinality ce) {
         // optional, nicht erforderlich zum Bestehen der bung
            return ce;
        }
    
        @Override
        public OWLClassExpression visit(OWLDataExactCardinality ce) {
         // optional, nicht erforderlich zum Bestehen der bung
            return ce;
        }
    
        @Override
        public OWLClassExpression visit(OWLDataMaxCardinality ce) {
         // optional, nicht erforderlich zum Bestehen der bung
            return ce;
        }
    }
}
