package de.uniulm.semweb.test;

import static org.junit.Assert.assertTrue;

import org.junit.Test;
import org.semanticweb.owlapi.apibinding.OWLManager;
import org.semanticweb.owlapi.model.IRI;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLClassExpression;
import org.semanticweb.owlapi.model.OWLDataFactory;
import org.semanticweb.owlapi.model.OWLObjectProperty;

import de.uniulm.semweb.NNFConverter;

public class TestNNFConverter {

    protected OWLDataFactory dataFactory=OWLManager.getOWLDataFactory();
    protected OWLClass c=dataFactory.getOWLClass(IRI.create("http://ex.org/C"));
    protected OWLClass d=dataFactory.getOWLClass(IRI.create("http://ex.org/D"));
    protected OWLClass e=dataFactory.getOWLClass(IRI.create("http://ex.org/E"));
    protected OWLObjectProperty r=dataFactory.getOWLObjectProperty(IRI.create("http://ex.org/r"));
    protected NNFConverter converter=new NNFConverter();
    
    protected void doTest(OWLClassExpression toTest, OWLClassExpression expected) {
        OWLClassExpression result=converter.convertToNNF(toTest);
        assertTrue("Die NNF von "+toTest+"\nsollte "+expected+" sein,\naber war "+result+".", result.equals(expected));
    }
    
    @Test
    public void testConvertToNNF1() {
        OWLClassExpression toTest=c;
        OWLClassExpression expected=c;
        doTest(toTest, expected);
    }
    @Test
    public void testConvertToNNF2() {
        OWLClassExpression toTest=dataFactory.getOWLObjectComplementOf(c);
        OWLClassExpression expected=toTest;
        doTest(toTest, expected);
    }
    @Test
    public void testConvertToNNF3() {
        OWLClassExpression negC=dataFactory.getOWLObjectComplementOf(c);
        OWLClassExpression toTest=dataFactory.getOWLObjectComplementOf(negC);
        OWLClassExpression expected=c; 
        doTest(toTest, expected);
    }
    @Test
    public void testConvertToNNF4() {
        OWLClassExpression negC=dataFactory.getOWLObjectComplementOf(c);
        OWLClassExpression negNegC=dataFactory.getOWLObjectComplementOf(negC);
        OWLClassExpression toTest=dataFactory.getOWLObjectComplementOf(negNegC);
        OWLClassExpression expected=negC; 
        doTest(toTest, expected);
    }
    @Test
    public void testConvertToNNF5() {
        OWLClassExpression toTest=dataFactory.getOWLObjectIntersectionOf(c, d);
        OWLClassExpression expected=toTest;
        doTest(toTest, expected);
    }
    @Test
    public void testConvertToNNF6() {
        OWLClassExpression and=dataFactory.getOWLObjectIntersectionOf(c, d);
        OWLClassExpression toTest=dataFactory.getOWLObjectComplementOf(and);
        OWLClassExpression expected=dataFactory.getOWLObjectUnionOf(dataFactory.getOWLObjectComplementOf(c), dataFactory.getOWLObjectComplementOf(d));
        doTest(toTest, expected);
    }
    @Test
    public void testConvertToNNF7() {
        OWLClassExpression or=dataFactory.getOWLObjectUnionOf(c, dataFactory.getOWLObjectComplementOf(d));
        OWLClassExpression toTest=dataFactory.getOWLObjectComplementOf(or);
        OWLClassExpression expected=dataFactory.getOWLObjectIntersectionOf(dataFactory.getOWLObjectComplementOf(c), d);
        doTest(toTest, expected);
    }
    @Test
    public void testConvertToNNF8() {
        OWLClassExpression toTest=dataFactory.getOWLObjectSomeValuesFrom(r, c);
        OWLClassExpression expected=toTest;
        doTest(toTest, expected);
    }
    @Test
    public void testConvertToNNF9() {
        OWLClassExpression allRC=dataFactory.getOWLObjectAllValuesFrom(r, c);
        OWLClassExpression toTest=dataFactory.getOWLObjectComplementOf(allRC);
        OWLClassExpression expected=dataFactory.getOWLObjectSomeValuesFrom(r, dataFactory.getOWLObjectComplementOf(c));
        doTest(toTest, expected);
    }
    @Test
    public void testConvertToNNF10() {
        OWLClassExpression allRnotC=dataFactory.getOWLObjectAllValuesFrom(r, dataFactory.getOWLObjectComplementOf(c));
        OWLClassExpression toTest=dataFactory.getOWLObjectComplementOf(allRnotC);
        OWLClassExpression expected=dataFactory.getOWLObjectSomeValuesFrom(r, c);
        doTest(toTest, expected);
    }
    @Test
    public void testConvertToNNF11() {
        OWLClassExpression negD=dataFactory.getOWLObjectComplementOf(d);
        OWLClassExpression negE=dataFactory.getOWLObjectComplementOf(e);
        OWLClassExpression somerNegE=dataFactory.getOWLObjectSomeValuesFrom(r, negE);
        OWLClassExpression negSomerNegE=dataFactory.getOWLObjectComplementOf(somerNegE);
        OWLClassExpression negNegD=dataFactory.getOWLObjectComplementOf(negD);
        OWLClassExpression allrNegNegD=dataFactory.getOWLObjectAllValuesFrom(r, negNegD);
        OWLClassExpression negAllrNegNegD=dataFactory.getOWLObjectComplementOf(allrNegNegD);
        OWLClassExpression and=dataFactory.getOWLObjectIntersectionOf(c, negAllrNegNegD);
        OWLClassExpression or=dataFactory.getOWLObjectUnionOf(and, negSomerNegE);
        OWLClassExpression toTest=dataFactory.getOWLObjectComplementOf(or);
        OWLClassExpression orNnf=dataFactory.getOWLObjectUnionOf(dataFactory.getOWLObjectComplementOf(c), dataFactory.getOWLObjectAllValuesFrom(r, d));
        OWLClassExpression expected=dataFactory.getOWLObjectIntersectionOf(orNnf, somerNegE);
        doTest(toTest, expected);
    }

}
